################################################
################################################
# This is script will guide you through the    #
# analysis of the controlled NH3 release       #
# experiment discussed in the article:         #
#  "Accounting for field-scale dry deposition  #
# in backward Lagrangian stochastic dispersion # 
#         modelling of NH3 emissions"          #
################################################
################################################


########################
### 0. Script Header ###
########################
	
	# load bLSmodelR package
	library(bLSmodelR)

	### edit directory paths
	# *edit* path to data files directory
	dirPath <- "U:/Documents/7_Papers/Paper_bLS/Paper_Deposition/Supplement S1 - bLSmodelR and Data"
	# *edit* path to save touchdown catalogs
	CatPath <- getwd() 

	### set parameters
	# number of CPU cores for parallelization
	ncores <- 1 
	# ncores <- 4 
	# number of trajectories (evaluation presented in paper: N0 = 2.5E6)
	N0 <- 5E4
	# N0 <- 2.5E6
	# distance (in m) between point sensors in line sensor approximation  (evaluation presented in paper: dpoint = 0.5)
	dpoint <- 1
	# dpoint <- 0.5
	
########################

##########################
### 1. Set Up bLS Run  ###
##########################
	
	### read data
	# Field_Data.csv
	Field_Data <- read.table(paste(dirPath,"Field_Data.csv",sep="/"),sep = ";",header = TRUE,check.names = FALSE,stringsAsFactors = FALSE)
	# fix dates:
	for(i in 1:2)if(is.character(Field_Data[[i]]))Field_Data[[i]] <- as.POSIXct(strptime(Field_Data[[i]],format="%d.%m.%Y %H:%M",tz="Etc/GMT-1"))
	# Geometry.csv
	Geometry <- read.table(paste(dirPath,"Geometry.csv",sep="/"),sep = ";",header = TRUE,check.names = FALSE,stringsAsFactors = FALSE)
	# center coordinates
	Geometry[,2:3] <- sweep(Geometry[,2:3],2,colMeans(Geometry[grep("Orifice",Geometry[,1]),2:3]))
	

	### set up source geometry
	# get orifice coordinates
	Orifices <- Geometry[grep("Orifice",Geometry[,1]),2:3]
	# prepare input list for genSources
	OrifList <- apply(Orifices,1,function(x)list("c",M=x,R=0.05))
	names(OrifList)[] <- "Artif. Source"
	# create Sources object
	ArtifSource <- genSources(OrifList) 
	# check
	ArtifSource
	plot(ArtifSource)
	
	
	### set up sensor geometry
	# get sensor coordinates
	Sensors <- Geometry[grep("DOAS",Geometry[,1]),]
	# prepare input data.frame for genSensors
	SensDFrame <- cbind(
		PositionName = gsub("DOAS ","",Sensors[[1]])
		,Sensors[,2:4]
		,SensorName = gsub(".* (.*) .*","\\1",Sensors[[1]])
		,N = NA
		,d = dpoint
		,stringsAsFactors = FALSE)
	# create Sensors object
	miniDOAS <- genSensors(SensDFrame)
	# check
	miniDOAS
	plot(miniDOAS,ArtifSource)
	
	
	### prepare interval data (displacement height = 6.4 cm)
	IntData <- cbind(Field_Data
		,d = 0.064
		,N0 = N0
		# ,MaxFetch = 200 # original analysis used 200 m (for plotting reasons)
		,MaxFetch = -20
		,z_sWu = Geometry[grep("sonic anemometer",Geometry[,1]),4])
	# set names (see args(genInterval))
	names(IntData)[3:9] <- c("Ustar","Zo","L","sUu","sVu","sWu","WD")
	# create Interval object
	Ints <- genInterval(IntData)


	### run bLS model
	# prepare input list
	InList <- genInputList(ArtifSource,miniDOAS,Ints)
	# check
	InList
	plot(InList)
	# run the bLS model
	Results <- runbLS(InList,CatPath,ncores = ncores)
	# check
	par(mfrow=c(2,2),oma=c(0,0.25,0,0))
	Results[,{
		plot(CE~rn,type="b",lty=3,main=.BY[[1]],xlab=""
			,ylab=expression(group("{","C/Q","}")["sim"]*" [s m"^{-1}*"]"))
	},by=Sensor]
	# check number of TD inside source (should be approx. 1E3 or more)
	Results[,N_TD]

##########################

#####################################
### 2. Deposition post-processing ###
#####################################
	
	### run deposition post-processing on bLS results
	# prepare deposition velocity values
	Rc <- seq(0,500,50)
	Results[,paste0("vDep",Rc) := as.data.table(1/outer(get("Rb [s/m]"),Rc,"+"))]
	# run dry deposition post-processing
	ResDep <- vector("list",length(Rc))
	names(ResDep) <- paste0("vDep",Rc)
	for(i in names(ResDep))ResDep[[i]] <- deposition(Results,vDep = i,ncores = ncores)

	--> load workspace here
	
	original TD catalogs auf switchdrive stellen und link?!!!

#####################################


#######################################
### 3. Evaluate Recovered Fractions ###
#######################################
	
	### release rate
	# total release rate in ug/s
	Qtot <- 101.3*10/8.314/273*17*0.0497/60*10^6 # 10 nL/min, 4.97%: p_n*V/R/T_n*M_NH3*Xi_%/secs*mins -> ug/s
	# release rate in ug/m2/s
	Qsrc <- Qtot/getArea(ArtifSource)
	# sort delta NH3:
	Results[,dNH3 := NA_real_][,dNH3 := get(paste0(.BY[[1]],": deltaNH3 [ug/m3]")),by=Sensor]
	
	### recovered fractions excluding dry deposition
	# calculate recovered fraction (Rf) without dry deposition
	Results[,":="(
		Rf = dNH3/CE/Qsrc
		,st = get("Start of interval (GMT+1)")
		,et = get("End of interval (GMT+1)")
		)]
	# visualize
	par(mfrow=c(2,2))
	Results[,{
		mt <- st + (et - st)/2
		plot(Rf ~ mt,type="n",main=.BY[[1]],xlab=""
			,ylab=expression(paste(Q["bLS"],"/Q [-]")),ylim=c(0,2))
		abline(h=1,col="darkgrey")
		lines(Rf ~ mt,type="b",lty=3)
	},by=Sensor]

	### recovered fractions including dry deposition
	# prepare deposition results
	Results_inclDep <- rbind(
		rbindlist(mapply(function(x,y){
			x[,dNH3 := NA_real_][,dNH3 := get(paste0(.BY[[1]],": deltaNH3 [ug/m3]")),by=Sensor]
			x[,.(
				rn
				,Sensor
				,Rc = y
				,Rb = get("Rb [s/m]")
				,vDep = get(paste0("vDep",y))
				,RfDep = dNH3/CE_Dep/Qsrc
				,st = get("Start of interval (GMT+1)")
				,et = get("End of interval (GMT+1)")
				)]
		},x=ResDep,y=Rc,SIMPLIFY = FALSE))
		,Results[,.(rn,Sensor,Rc = Inf,Rb = get("Rb [s/m]"),vDep = 0,RfDep = dNH3/CE/Qsrc,st,et)])

	### estimate best fitting vDep/Rc using loess
	BestRc <- Results_inclDep[,{
		mod <- loess(RfDep~vDep)
		xpred <- seq(0,max(mod$x),length.out=1E5)
		pmod <- predict(mod,newdata=data.frame(vDep=xpred))
		ind <- which.min(abs(pmod - 1))
		.(
			vDep = xpred[ind]
			,Rc = round(1/xpred[ind] - Rb[1])
			,Recov = pmod[ind]
			,st = st[1]
			,et = et[1]
		)
	  },by=.(Sensor,rn)]
	  # visualize
	  par(mfrow=c(2,2))
	  BestRc[,{
	  	plot(vDep ~ I(st + (et - st)/2),type="b",lty=3,main=.BY[[1]],xlab="")
	  },by=Sensor]

#######################################



